/// <reference types="node" />
import EventEmitter from 'events';
import type { MessageEntity, Params } from 'typegram';
import type { CallbackQueryRequest, ClientOptions, CommandRequest, MessageRequest } from './modules/telegramClient';
import { TelegramClient } from './modules/telegramClient';
interface WebHook {
    url: string;
}
interface StoredUpdate {
    time: number;
    botToken: string;
    updateId: number;
    messageId: number;
    isRead: boolean;
}
declare type BotIncommingMessage = Params<'sendMessage', never>[0] & {
    reply_markup?: string;
};
declare type BotEditTextIncommingMessage = Params<'editMessageText', never>[0] & {
    reply_markup?: string;
};
export interface StoredBotUpdate extends StoredUpdate {
    message: BotIncommingMessage;
}
interface StoredMessageUpdate extends StoredUpdate {
    message: MessageRequest;
}
interface StoredCommandUpdate extends StoredUpdate {
    message: CommandRequest;
    entities: MessageEntity[];
}
interface StoredCallbackQueryUpdate extends StoredUpdate {
    callbackQuery: CallbackQueryRequest;
    callbackId: number;
}
export declare type StoredClientUpdate = StoredMessageUpdate | StoredCommandUpdate | StoredCallbackQueryUpdate;
interface Storage {
    userMessages: StoredClientUpdate[];
    botMessages: StoredBotUpdate[];
}
export interface TelegramServerConfig {
    port: number;
    host: string;
    protocol: 'http' | 'https';
    storage: 'RAM' | string;
    storeTimeout: number;
}
export declare class TelegramServer extends EventEmitter {
    private webServer;
    private started;
    private updateId;
    private messageId;
    private callbackId;
    config: TelegramServerConfig & {
        apiURL: string;
    };
    storage: Storage;
    private cleanUpDaemonInterval;
    private server;
    private webhooks;
    constructor(config?: Partial<TelegramServerConfig>);
    static normalizeConfig(config: Partial<TelegramServerConfig>): {
        port: number;
        host: string;
        protocol: "http" | "https";
        storage: string;
        storeTimeout: number;
        apiURL: string;
    };
    getClient(botToken: string, options?: Partial<ClientOptions>): TelegramClient;
    addBotMessage(message: BotIncommingMessage, botToken: string): void;
    editMessageText(message: BotEditTextIncommingMessage): void;
    waitBotEdits(): Promise<void>;
    waitBotMessage(): Promise<void>;
    waitUserMessage(): Promise<void>;
    addUserMessage(message: MessageRequest): Promise<void>;
    addUserCommand(message: CommandRequest): Promise<void>;
    addUserCallback(callbackQuery: CallbackQueryRequest): Promise<void>;
    private getCommonFields;
    private addUserUpdate;
    messageFilter(message: StoredUpdate): boolean;
    cleanUp(): void;
    cleanUpDaemon(): void;
    getUpdatesHistory(token: string): (StoredBotUpdate | StoredMessageUpdate | StoredCallbackQueryUpdate)[];
    getUpdates(token: string): ({
        update_id: number;
        callback_query: {
            id: string;
            from: import("typegram").User;
            message: import("./modules/telegramClient").CommonMessage;
            data: string;
        };
        message?: undefined;
    } | {
        update_id: number;
        message: {
            entities?: MessageEntity[] | undefined;
            message_id: number;
            from: import("typegram").User;
            chat: import("typegram").Chat;
            date: number;
            text: string;
        };
        callback_query?: undefined;
    })[];
    start(): Promise<void>;
    removeUserMessage(updateId: number): void;
    removeBotMessage(updateId: number): void;
    setWebhook(webhook: WebHook, botToken: string): void;
    deleteWebhook(botToken: string): void;
    deleteMessage(chatId: number, messageId: number): boolean;
    stop(): Promise<boolean>;
    private formatUpdate;
}
export {};
